@@ -61,9 +61,11 @@ GT_INLINE void gt_alignment_delete(gt_alignment* const alignment);
  */
 GT_INLINE char* gt_alignment_get_tag(gt_alignment* const alignment);
 GT_INLINE void gt_alignment_set_tag(gt_alignment* const alignment,char* const tag,const uint64_t length);
+GT_INLINE uint64_t gt_alignment_get_tag_length(gt_alignment* const alignment);
 
 GT_INLINE char* gt_alignment_get_read(gt_alignment* const alignment);
 GT_INLINE void gt_alignment_set_read(gt_alignment* const alignment,char* const read,const uint64_t length);
+GT_INLINE uint64_t gt_alignment_get_read_length(gt_alignment* const alignment);
 
 GT_INLINE char* gt_alignment_get_qualities(gt_alignment* const alignment);
 GT_INLINE void gt_alignment_set_qualities(gt_alignment* const alignment,char* const qualities,const uint64_t length);
@@ -111,9 +113,7 @@ GT_INLINE bool gt_alignment_locate_map_reference(gt_alignment* const alignment,g
 /*
  * Miscellaneous
  */
-GT_INLINE void gt_alignment_handler_copy(gt_alignment* const alignment_dst,gt_alignment* const alignment_src);
-GT_INLINE gt_alignment* gt_alignment_copy(gt_alignment* const alignment);
-GT_INLINE gt_alignment* gt_alignment_deep_copy(gt_alignment* const alignment);
+GT_INLINE gt_alignment* gt_alignment_copy(gt_alignment* const alignment,const bool copy_maps);
 
 // Alignment's Maps iterator
 GT_INLINE void gt_alignment_new_map_iterator(gt_alignment* const alignment,gt_alignment_map_iterator* const alignment_map_iterator);
