@@ -116,6 +116,7 @@ typedef int32_t gt_status;
 #define GT_MIN(a,b) ((a)<=(b)?(a):(b))
 #define GT_MAX(a,b) ((a)>=(b)?(a):(b))
 #define GT_ABS(a) ((a)>=0?(a):-(a))
+#define GT_SWAP(a,b) do {typeof(a) aux = a; a = b; b = aux; } while (0)
 
 /*
  * String/Buffer functions
@@ -127,18 +128,30 @@ GT_INLINE bool gt_streq(char* const buffer_a,char* const buffer_b);
 GT_INLINE int gt_strncmp(char* const buffer_a,char* const buffer_b,const uint64_t length);
 GT_INLINE bool gt_strneq(char* const buffer_a,char* const buffer_b,const uint64_t length);
 
+/*
+ * Memory usage functions
+ */
+GT_INLINE uint64_t gt_calculate_memory_required_v(const char *template,va_list v_args);
+GT_INLINE uint64_t gt_calculate_memory_required_va(const char *template,...);
+
 /*
  * Error value return wrapper
  */
 #define GT_DELEGATE_ERROR(funtion_call,error_code) if ((error_code=funtion_call)) { return error_code; }
 
 /*
- * Mutex Helpers
+ * Mutex/Cond Helpers
  */
 #define GT_BEGIN_MUTEX_SECTION(mutex) \
   gt_cond_fatal_error(pthread_mutex_lock(&(mutex)),SYS_MUTEX);
 #define GT_END_MUTEX_SECTION(mutex) \
   gt_cond_fatal_error(pthread_mutex_unlock(&(mutex)),SYS_MUTEX);
+#define GT_CV_SIGNAL(cv) \
+  gt_cond_fatal_error(pthread_cond_signal(&(cv)),SYS_COND_VAR);
+#define GT_CV_BROADCAST(cv) \
+  gt_cond_fatal_error(pthread_cond_broadcast(&(cv)),SYS_COND_VAR);
+#define GT_CV_WAIT(cv,mutex) \
+  gt_cond_fatal_error(pthread_cond_wait(&(cv),&(mutex)),SYS_COND_VAR);
 
 /*
  * GEM-Tools basic includes
@@ -153,5 +166,6 @@ GT_INLINE bool gt_strneq(char* const buffer_a,char* const buffer_b,const uint64_
  */
 #define GT_STREAM_FILE_NAME "<<STREAM>>"
 #define GT_ALL UINT64_MAX
+#define GT_NO_STRATA ((int64_t)(-1))
 
 #endif /* GT_COMMONS_H_ */
