@@ -160,7 +160,10 @@ extern FILE* gt_error_stream;
 // Output errors
 #define GT_ERROR_FPRINTF "Printing output. 'fprintf' call failed"
 #define GT_ERROR_SPRINTF "Printing output. 'sprintf' call failed"
-#define GT_ERROR_BPRINTF "Printing output. Buffered print formated call failed"
+#define GT_ERROR_BPRINTF "Printing output. Buffer print formated 'gt_bprintf' call failed"
+#define GT_ERROR_OFPRINTF "Printing output. Output File print formated 'gt_ofprintf' call failed"
+#define GT_ERROR_BOFPRINTF "Printing output. Buffered Output file print formated 'gt_bofprintf' call failed"
+
 #define GT_ERROR_PRINT_FORMAT "Incorrect print format. Expected format character"
 
 // Template/Alignment/Map/Misms errors
@@ -176,6 +179,7 @@ extern FILE* gt_error_stream;
 #define GT_ERROR_TEMPLATE_INCONSISTENT_MMAPS_ALIGNMENT "Template inconsistency. Multimaps' members must be contained by single alignments"
 #define GT_ERROR_TEMPLATE_INCONSISTENT_MMAPS_ATTRB_RELATION "Template inconsistency. Incorrect number of mmaps and mmaps' attributes"
 #define GT_ERROR_TEMPLATE_INCONSISTENT_NUM_MAPS_RELATION "Template inconsistency. Incorrect number of matches' elements (check num_blocks_template)"
+#define GT_ERROR_TEMPLATE_INCONSISTENT_NUM_BLOCKS "Template inconsistency. Number of blocks must be the same across templates"
 #define GT_ERROR_TEMPLATE_ADD_BAD_NUM_BLOCKS "Trying to add wrong number of blocks to the template"
 #define GT_ERROR_PALIGN_BAD_NUM_BLOCKS "Invalid Paired-alignment. Wrong number of alignment blocks (%"PRIu64")"
 
