@@ -51,13 +51,13 @@ GT_INLINE void gt_alignment_clear(gt_alignment* const alignment) {
 }
 GT_INLINE void gt_alignment_delete(gt_alignment* const alignment) {
   GT_ALIGNMENT_CHECK(alignment);
-  gt_alignment_clear_maps(alignment);
   gt_string_delete(alignment->tag);
   gt_string_delete(alignment->read);
   gt_string_delete(alignment->qualities);
   gt_vector_delete(alignment->counters);
-  gt_vector_delete(alignment->maps);
+  gt_alignment_clear_maps(alignment);
   gt_shash_delete(alignment->maps_dictionary,true,true);
+  gt_vector_delete(alignment->maps);
   gt_shash_delete(alignment->attributes,true,true);
   free(alignment);
 }
@@ -74,6 +74,10 @@ GT_INLINE void gt_alignment_set_tag(gt_alignment* const alignment,char* const ta
   GT_NULL_CHECK(tag);
   gt_string_set_nstring(alignment->tag,tag,length);
 }
+GT_INLINE uint64_t gt_alignment_get_tag_length(gt_alignment* const alignment) {
+  GT_ALIGNMENT_CHECK(alignment);
+  return gt_string_get_length(alignment->tag);
+}
 GT_INLINE char* gt_alignment_get_read(gt_alignment* const alignment) {
   GT_ALIGNMENT_CHECK(alignment);
   return gt_string_get_string(alignment->read);
@@ -85,6 +89,10 @@ GT_INLINE void gt_alignment_set_read(gt_alignment* const alignment,char* const r
   gt_fatal_check(!gt_string_is_null(alignment->qualities) &&
       gt_string_get_length(alignment->qualities)!=gt_string_get_length(alignment->read),ALIGN_READ_QUAL_LENGTH);
 }
+GT_INLINE uint64_t gt_alignment_get_read_length(gt_alignment* const alignment) {
+  GT_ALIGNMENT_CHECK(alignment);
+  return gt_string_get_length(alignment->read);
+}
 GT_INLINE char* gt_alignment_get_qualities(gt_alignment* const alignment) {
   GT_ALIGNMENT_CHECK(alignment);
   return gt_string_get_string(alignment->qualities);
@@ -272,31 +280,26 @@ GT_INLINE void gt_alignment_handler_copy(gt_alignment* const alignment_dst,gt_al
   GT_ALIGNMENT_CHECK(alignment_src);
   alignment_dst->alignment_id = alignment_src->alignment_id;
   alignment_dst->in_block_id = alignment_src->in_block_id;
-  alignment_dst->tag = gt_string_copy(alignment_src->tag);
-  alignment_dst->read = gt_string_copy(alignment_src->read);
-  alignment_dst->qualities = gt_string_copy(alignment_src->qualities);
+  alignment_dst->tag = gt_string_dup(alignment_src->tag);
+  alignment_dst->read = gt_string_dup(alignment_src->read);
+  alignment_dst->qualities = gt_string_dup(alignment_src->qualities);
   alignment_dst->maps_txt = alignment_src->maps_txt;
 }
-GT_INLINE gt_alignment* gt_alignment_copy(gt_alignment* const alignment) {
+GT_INLINE gt_alignment* gt_alignment_copy(gt_alignment* const alignment,const bool copy_maps) {
   GT_ALIGNMENT_CHECK(alignment);
   gt_alignment* alignment_cp = gt_alignment_new();
   gt_cond_fatal_error(!alignment_cp,MEM_HANDLER);
   // Copy handler
   gt_alignment_handler_copy(alignment_cp,alignment);
-  return alignment_cp;
-}
-GT_INLINE gt_alignment* gt_alignment_deep_copy(gt_alignment* const alignment) {
-  GT_ALIGNMENT_CHECK(alignment);
-  gt_alignment* alignment_cp = gt_alignment_new();
-  gt_cond_fatal_error(!alignment_cp,MEM_HANDLER);
-  // Copy handler
-  gt_alignment_handler_copy(alignment_cp,alignment);
-  // Copy attributes
-  alignment_cp->attributes = gt_shash_deep_copy(alignment->attributes);
-  // Copy map related fields (deep copy) {MAPS,MAPS_DICCTIONARY,COUNTERS,ATTRIBUTES}
-  gt_vector_copy(alignment_cp->counters,alignment->counters);
-  GT_VECTOR_ITERATE(alignment->maps,alg_map,alg_map_pos,gt_map*) {
-    gt_alignment_add_map(alignment_cp,gt_map_copy(*alg_map));
+  // Copy maps
+  if (copy_maps) {
+    // Copy attributes
+    alignment_cp->attributes = gt_shash_deep_copy(alignment->attributes);
+    // Copy map related fields (deep copy) {MAPS,MAPS_DICCTIONARY,COUNTERS,ATTRIBUTES}
+    gt_vector_copy(alignment_cp->counters,alignment->counters);
+    GT_VECTOR_ITERATE(alignment->maps,alg_map,alg_map_pos,gt_map*) {
+      gt_alignment_add_map(alignment_cp,gt_map_copy(*alg_map));
+    }
   }
   return alignment_cp;
 }
