@@ -12,8 +12,6 @@
 /*
  * Useful macros
  */
-#define GT_IMP_PARAMS_FILE__LINE(buffered_input_file) \
-  buffered_input_file->input_file->file_name,buffered_input_file->current_line_num
 // MAP/MAPQ related
 #define gt_is_valid_template_separator(character) \
   (character==' ')
@@ -132,15 +130,16 @@ GT_INLINE gt_status gt_input_map_parser_check_map_file_format(gt_buffered_input_
 }
 
 /*
- * MAP file. Error handler
+ * MAP File basics
  */
+/* Error handler */
 GT_INLINE void gt_input_map_parser_prompt_error(
-    gt_buffered_input_file* const buffered_input_file,
+    gt_buffered_input_file* const buffered_map_input,
     uint64_t line_num,uint64_t column_pos,const gt_status error_code) {
   // Display textual error msg
-  register const char* const file_name = (buffered_input_file != NULL) ?
-      buffered_input_file->input_file->file_name : "<<LazyParsing>>";
-  if ((buffered_input_file == NULL)) {
+  register const char* const file_name = (buffered_map_input != NULL) ?
+      buffered_map_input->input_file->file_name : "<<LazyParsing>>";
+  if ((buffered_map_input == NULL)) {
     line_num = 0; column_pos = 0;
   }
   switch (error_code) {
@@ -167,23 +166,73 @@ GT_INLINE void gt_input_map_parser_prompt_error(
     case GT_IMP_PE_MISMS_BAD_CHARACTER: gt_error(PARSE_MAP_MISMS_BAD_CHARACTER,file_name,line_num,column_pos); break;
     case GT_IMP_PE_MISMS_BAD_MISMS_POS: gt_error(PARSE_MAP_MISMS_BAD_MISMS_POS,file_name,line_num,column_pos); break;
     default:
-      gt_error(PARSE_MAP,buffered_input_file->input_file->file_name,line_num);
+      gt_error(PARSE_MAP,buffered_map_input->input_file->file_name,line_num);
       break;
   }
 }
-/*
- * MAP file. Skip record
- */
-GT_INLINE void gt_input_map_parser_next_record(gt_buffered_input_file* const buffered_input_file) {
-  GT_BUFFERED_INPUT_FILE_CHECK(buffered_input_file);
-  if (!gt_buffered_input_file_eob(buffered_input_file)) {
-    GT_INPUT_FILE_SKIP_LINE(buffered_input_file);
+/* MAP file. Skip record */
+GT_INLINE void gt_input_map_parser_next_record(gt_buffered_input_file* const buffered_map_input) {
+  GT_BUFFERED_INPUT_FILE_CHECK(buffered_map_input);
+  if (!gt_buffered_input_file_eob(buffered_map_input)) {
+    GT_INPUT_FILE_SKIP_LINE(buffered_map_input);
   }
 }
+/* MAP file. Synchronized get block wrt to paired map records */
+GT_INLINE gt_status gt_input_map_parser_get_block(
+    gt_buffered_input_file* const buffered_map_input,const uint64_t num_records) {
+  GT_BUFFERED_INPUT_FILE_CHECK(buffered_map_input);
+  register gt_input_file* const input_file = buffered_map_input->input_file;
+  // Read lines
+  if (input_file->eof) return GT_BMI_EOF;
+  gt_input_file_lock(input_file);
+  if (input_file->eof) {
+    gt_input_file_unlock(input_file);
+    return GT_BMI_EOF;
+  }
+  buffered_map_input->block_id = gt_input_file_next_id(input_file) % UINT32_MAX;
+  buffered_map_input->current_line_num = input_file->processed_lines+1;
+  gt_vector_clean(buffered_map_input->block_buffer); // Clear dst buffer
+  // Read lines
+  uint64_t lines_read = 0, num_blocks = 0;
+  while ( (lines_read<num_records || num_blocks%2!=0) &&
+      gt_input_file_next_map_record(input_file,buffered_map_input->block_buffer,&num_blocks) ) ++lines_read;
+  // Dump remaining content into the buffer
+  gt_input_file_dump_to_buffer(input_file,buffered_map_input->block_buffer);
+  if (lines_read > 0 && *gt_vector_get_last_elm(buffered_map_input->block_buffer,char) != EOL) {
+    gt_vector_insert(buffered_map_input->block_buffer,EOL,char);
+  }
+  input_file->processed_lines+=lines_read;
+  buffered_map_input->lines_in_buffer = lines_read;
+  gt_input_file_unlock(input_file);
+  // Setup the block
+  buffered_map_input->cursor = gt_vector_get_mem(buffered_map_input->block_buffer,char);
+  return buffered_map_input->lines_in_buffer;
+}
+/* MAP file. Reload internal buffer */
+GT_INLINE gt_status gt_input_map_parser_reload_buffer(
+    gt_buffered_input_file* const buffered_map_input,const bool synchronized_map) {
+  GT_BUFFERED_INPUT_FILE_CHECK(buffered_map_input);
+  // Dump buffer if BOF it attached to Map-input, and get new out block (always FIRST)
+  if (buffered_map_input->buffered_output_file!=NULL) {
+    gt_buffered_output_file_dump(buffered_map_input->buffered_output_file);
+  }
+  // Read new input block
+  register const uint64_t read_lines = (synchronized_map) ?
+      gt_input_map_parser_get_block(buffered_map_input,GT_IMP_NUM_LINES):
+      gt_buffered_input_file_get_block(buffered_map_input,GT_IMP_NUM_LINES);
+  if (gt_expect_false(read_lines==0)) return GT_IMP_EOF;
+  // Assign block ID
+  if (buffered_map_input->buffered_output_file!=NULL) {
+    gt_buffered_output_file_set_block_ids(
+        buffered_map_input->buffered_output_file,buffered_map_input->block_id,0);
+  }
+  return GT_IMP_OK;
+}
+
 /*
  * MAP format. Basic building block for parsing
  */
-GT_INLINE gt_status gt_input_map_parse_tag(char** const text_line,gt_string* const tag) {
+GT_INLINE gt_status gt_input_map_parse_tag(char** const text_line,gt_string* const tag) { // FIXME: read all
   // Delimit the tag
   register char* const tag_begin = *text_line;
   GT_READ_UNTIL(text_line,**text_line==TAB||**text_line==SPACE);
@@ -205,24 +254,6 @@ GT_INLINE gt_status gt_input_map_parse_tag(char** const text_line,gt_string* con
   }
   return 0;
 }
-GT_INLINE gt_status gt_input_map_parse_tag_block(char** tag,char** tag_block,const uint64_t num_expected_blocks) {
-  /*
-   * Tag Block Conventions::
-   *  (1) line1: NAME => line1&2: NAME
-   *      line2: NAME
-   *  (2) line1: NAME<SEP>SOMETHING_ELSE_1 => line1&2: NAME<SEP>SOMETHING_ELSE_1|NAME<SEP>SOMETHING_ELSE_2
-   *      line2: NAME<SEP>SOMETHING_ELSE_2
-   *  (3) line1: NAME<SEP>1 => line1&2: NAME
-   *      line2: NAME<SEP>2
-   * Tag Decomposition::
-   *  (<TAG_BLOCK><SEP>){num_blocks}
-   *    -> tag.num_blocks == read.num_blocks
-   *    -> <SEP> := ' ' | '/' | '#' | "|"  // TODO
-   */
-
-  // TODO
-  return 0;
-}
 GT_INLINE gt_status gt_input_map_parse_read_block(char** const text_line,gt_string* const read_block) {
   // Read READ_BLOCK
   register char* const read_block_begin = *text_line;
@@ -283,7 +314,7 @@ GT_INLINE gt_status gt_imp_counters(char** const text_line,gt_vector* const coun
     return 0;
   }
   // Parse counters
-  register bool prev_char_was_sep = false;
+  register bool prev_char_was_sep = false, is_mcs_set = false;
   register uint64_t number;
   while (gt_expect_true(**text_line!=TAB && **text_line!=EOL && **text_line!=EOS)) {
     if (gt_is_number(**text_line)) {
@@ -304,7 +335,8 @@ GT_INLINE gt_status gt_imp_counters(char** const text_line,gt_vector* const coun
         gt_vector_insert(counters,number,uint64_t);
       }
     } else if (**text_line==GT_MAP_MCS) {
-      if (prev_char_was_sep) return GT_IMP_PE_COUNTERS_BAD_CHARACTER;
+      if (prev_char_was_sep || is_mcs_set) return GT_IMP_PE_COUNTERS_BAD_CHARACTER;
+      is_mcs_set = true;
       gt_attribute_set(attributes,GT_ATTR_MAX_COMPLETE_STRATA,&(gt_vector_get_used(counters)),sizeof(uint64_t));
       GT_NEXT_CHAR(text_line);
       prev_char_was_sep = true;
@@ -317,6 +349,10 @@ GT_INLINE gt_status gt_imp_counters(char** const text_line,gt_vector* const coun
       return GT_IMP_PE_COUNTERS_BAD_CHARACTER;
     }
   }
+  // Set default MCS
+  if (!is_mcs_set) {
+    gt_attribute_set(attributes,GT_ATTR_MAX_COMPLETE_STRATA,&(gt_vector_get_used(counters)),sizeof(uint64_t));
+  }
   // Parse attributes (if any)
   if (**text_line==GT_MAP_COUNTS_SEP && *(*text_line+1)==GT_MAP_COUNTS_SEP) { // 0:0::<Value>::<Value>
     return GT_IMP_PE_NOT_IMPLEMENTED; // TODO
@@ -824,6 +860,7 @@ GT_INLINE gt_status gt_imp_parse_map(char** text_line,gt_map* const map,const gt
   (error_code!=GT_IMP_PE_MAP_PENDING_MAPS && \
    error_code!=GT_IMP_PE_EOB && \
    error_code!=GT_IMP_PE_MAP_GLOBAL_ATTR)
+
 GT_INLINE gt_status gt_imp_parse_template_maps(
     char** text_line,gt_template* const template,
     const gt_lazy_parse_mode parse_mode,uint64_t num_maps) {
@@ -885,7 +922,8 @@ GT_INLINE gt_status gt_imp_parse_template_maps(
     if (gt_expect_false(num_blocks_parsed<num_blocks_template || num_blocks_parsed==1)) {
       return GT_IMP_PE_MAP_BAD_NUMBER_OF_BLOCKS;
     }
-    gt_template_insert_mmap_gtvector(template,vector_maps,&mmap_attr,true);
+    gt_template_raw_put_mmap(gt_map_cmp,template,
+        gt_vector_get_mem(vector_maps,gt_map*),&mmap_attr);
     ++num_maps_parsed;
   }
   gt_vector_delete(vector_maps);
@@ -932,8 +970,12 @@ GT_INLINE gt_status gt_imp_parse_alignment_maps(
           if (gt_expect_false(split_maps!=NULL)) gt_vector_delete(split_maps);
           return GT_IMP_PE_MAP_BAD_NUMBER_OF_BLOCKS; /* FIXME: Weird case of split blocks */
         case GT_IMP_PE_MAP_GLOBAL_ATTR:
-          if (gt_expect_false(split_maps!=NULL)) gt_vector_delete(split_maps);
-          return GT_IMP_PE_MAP_BAD_CHARACTER;
+          /* FIXME: Paolo score for SE ::: instead of : */
+          // if (gt_expect_false(split_maps!=NULL)) gt_vector_delete(split_maps);
+          // return GT_IMP_PE_MAP_BAD_CHARACTER;
+          if (gt_expect_false(!gt_is_number((**text_line)))) return GT_IMP_PE_MAP_BAD_CHARACTER;
+          GT_PARSE_NUMBER(text_line,map->score);
+          break;
         case GT_IMP_PE_MAP_PENDING_MAPS:
         case GT_IMP_PE_EOB: break;
         default:
@@ -1005,8 +1047,12 @@ GT_INLINE gt_status gt_imp_parse_template(
     ++num_blocks;
   }
 
-  // Tag Splitting (try to deduce alignments' tag out of the one template's tag) // TODo
-  //gt_input_map_parse_tag_block(char** tag,char** tag_block,const uint64_t num_expected_blocks);
+  // TAG Setup {Tag Splitting/Swapping}
+  if (gt_expect_true(num_blocks>1)) {
+    gt_template_deduce_alignments_tags(template);
+  } else {
+    gt_string_copy(gt_template_get_block(template,0)->tag,template->tag);
+  }
 
   // QUALITIES
   if (has_map_quality) {
@@ -1024,7 +1070,13 @@ GT_INLINE gt_status gt_imp_parse_template(
   }
   // COUNTERS
   gt_vector_clean(gt_template_get_counters_vector(template));
-  if ((error_code=gt_imp_counters(text_line,gt_template_get_counters_vector(template),template->attributes))) return error_code;
+  if (gt_expect_true(num_blocks>1)) {
+    error_code=gt_imp_counters(text_line,gt_template_get_counters_vector(template),template->attributes);
+  } else {
+    register gt_alignment* const alignment = gt_template_get_block(template,0);
+    error_code=gt_imp_counters(text_line,gt_alignment_get_counters_vector(alignment),alignment->attributes);
+  }
+  if (error_code) return error_code;
   if (gt_expect_false((**text_line)!=TAB)) return GT_IMP_PE_PREMATURE_EOL;
   GT_NEXT_CHAR(text_line);
   // MAPS
@@ -1210,136 +1262,103 @@ GT_INLINE gt_status gt_input_map_parse_alignment_mismatch_string(gt_alignment* a
  * MAP High-level Parsers
  */
 GT_INLINE gt_status gt_imp_get_template(
-    gt_buffered_input_file* const buffered_input_file,gt_string* const src_text,
-    gt_template* const template,const gt_lazy_parse_mode parse_mode,
-    gt_output_buffer** const output_buffer) {
-  GT_BUFFERED_INPUT_FILE_CHECK(buffered_input_file);
+    gt_buffered_input_file* const buffered_map_input,gt_string* const src_text,
+    gt_template* const template,const gt_lazy_parse_mode parse_mode) {
+  GT_BUFFERED_INPUT_FILE_CHECK(buffered_map_input);
   GT_TEMPLATE_CHECK(template);
-  register gt_input_file* const input_file = buffered_input_file->input_file;
+  register gt_input_file* const input_file = buffered_map_input->input_file;
   register gt_status error_code;
   // Check the end_of_block. Reload buffer if needed
-  if (gt_buffered_input_file_eob(buffered_input_file)) {
-    // Dump buffer is Output it attached to Map-input
-//    if (output_buffer!=NULL && *output_buffer!=NULL && (*output_buffer)->buffered_output_file!=NULL) {
-//      *output_buffer = gt_buffered_output_file_dump_buffer(
-//          (*output_buffer)->buffered_output_file,*output_buffer);
-//    } // FIXME
-    register const uint64_t read_lines =
-        gt_buffered_input_file_get_block(buffered_input_file,GT_IMP_NUM_LINES,true);
-    if (gt_expect_false(read_lines==0)) return GT_IMP_EOF;
+  if (gt_buffered_input_file_eob(buffered_map_input)) {
+    if ((error_code=gt_input_map_parser_reload_buffer(buffered_map_input,true))!=GT_IMP_OK) return error_code;
   }
   // Check file format
-  if (gt_input_map_parser_check_map_file_format(buffered_input_file)) {
-    gt_error(PARSE_MAP_BAD_FILE_FORMAT,input_file->file_name,buffered_input_file->current_line_num);
+  if (gt_input_map_parser_check_map_file_format(buffered_map_input)) {
+    gt_error(PARSE_MAP_BAD_FILE_FORMAT,input_file->file_name,buffered_map_input->current_line_num);
     return GT_IMP_FAIL;
   }
   // Prepare the template
-  register char* const line_start = buffered_input_file->cursor;
-  register const uint64_t line_num = buffered_input_file->current_line_num;
+  register char* const line_start = buffered_map_input->cursor;
+  register const uint64_t line_num = buffered_map_input->current_line_num;
   gt_template_clear(template,true);
   template->template_id = line_num;
   // Parse template
-  if ((error_code=gt_imp_parse_template(&(buffered_input_file->cursor),
+  if ((error_code=gt_imp_parse_template(&(buffered_map_input->cursor),
       template,input_file->map_type.contains_qualities,parse_mode,UINT64_MAX))) {
-    gt_input_map_parser_prompt_error(buffered_input_file,line_num,
-        buffered_input_file->cursor-line_start,error_code);
-    gt_input_map_parser_next_record(buffered_input_file);
-    if (src_text) gt_string_set_nstring(src_text,line_start,buffered_input_file->cursor-line_start);
+    gt_input_map_parser_prompt_error(buffered_map_input,line_num,
+        buffered_map_input->cursor-line_start,error_code);
+    gt_input_map_parser_next_record(buffered_map_input);
+    if (src_text) gt_string_set_nstring(src_text,line_start,buffered_map_input->cursor-line_start);
     return GT_IMP_FAIL;
   }
   // Store source record
-  if (src_text) gt_string_set_nstring(src_text,line_start,buffered_input_file->cursor-line_start);
+  if (src_text) gt_string_set_nstring(src_text,line_start,buffered_map_input->cursor-line_start);
   // Next record
-  gt_input_map_parser_next_record(buffered_input_file);
+  gt_input_map_parser_next_record(buffered_map_input);
   return GT_IMP_OK;
 }
 GT_INLINE gt_status gt_imp_get_alignment(
-    gt_buffered_input_file* const buffered_input_file,gt_string* const src_text,
-    gt_alignment* const alignment,const gt_lazy_parse_mode parse_mode,
-    gt_output_buffer** const output_buffer) {
-  GT_BUFFERED_INPUT_FILE_CHECK(buffered_input_file);
+    gt_buffered_input_file* const buffered_map_input,gt_string* const src_text,
+    gt_alignment* const alignment,const gt_lazy_parse_mode parse_mode) {
+  GT_BUFFERED_INPUT_FILE_CHECK(buffered_map_input);
   GT_ALIGNMENT_CHECK(alignment);
-  register gt_input_file* const input_file = buffered_input_file->input_file;
+  register gt_input_file* const input_file = buffered_map_input->input_file;
   register gt_status error_code;
   // Check the end_of_block. Reload buffer if needed
-  if (gt_buffered_input_file_eob(buffered_input_file)) {
-    // Dump buffer is Output it attached to Map-input
-//    if (output_buffer!=NULL && *output_buffer!=NULL && (*output_buffer)->buffered_output_file!=NULL) {
-//      *output_buffer = gt_buffered_output_file_dump_buffer(
-//          (*output_buffer)->buffered_output_file,*output_buffer);
-//    } // FIXME
-    register const uint64_t read_lines =
-        gt_buffered_input_file_get_block(buffered_input_file,GT_IMP_NUM_LINES,true);
-    if (gt_expect_false(read_lines==0)) return GT_IMP_EOF;
+  if (gt_buffered_input_file_eob(buffered_map_input)) {
+    if ((error_code=gt_input_map_parser_reload_buffer(buffered_map_input,false))!=GT_IMP_OK) return error_code;
   }
   // Check file format
-  if (gt_input_map_parser_check_map_file_format(buffered_input_file)) {
-    gt_error(PARSE_MAP_BAD_FILE_FORMAT,input_file->file_name,buffered_input_file->current_line_num);
+  if (gt_input_map_parser_check_map_file_format(buffered_map_input)) {
+    gt_error(PARSE_MAP_BAD_FILE_FORMAT,input_file->file_name,buffered_map_input->current_line_num);
     return GT_IMP_FAIL;
   }
   // Allocate memory for the alignment
-  register char* const line_start = buffered_input_file->cursor;
-  register const uint64_t line_num = buffered_input_file->current_line_num;
+  register char* const line_start = buffered_map_input->cursor;
+  register const uint64_t line_num = buffered_map_input->current_line_num;
   gt_alignment_clear(alignment);
   alignment->alignment_id = line_num;
   // Parse alignment
-  if ((error_code=gt_imp_parse_alignment(&(buffered_input_file->cursor),
+  if ((error_code=gt_imp_parse_alignment(&(buffered_map_input->cursor),
       alignment,input_file->map_type.contains_qualities,parse_mode,UINT64_MAX))) {
-    gt_input_map_parser_prompt_error(buffered_input_file,line_num,
-        buffered_input_file->cursor-line_start,error_code);
-    gt_input_map_parser_next_record(buffered_input_file);
-    if (src_text) gt_string_set_nstring(src_text,line_start,buffered_input_file->cursor-line_start);
+    gt_input_map_parser_prompt_error(buffered_map_input,line_num,
+        buffered_map_input->cursor-line_start,error_code);
+    gt_input_map_parser_next_record(buffered_map_input);
+    if (src_text) gt_string_set_nstring(src_text,line_start,buffered_map_input->cursor-line_start);
     return GT_IMP_FAIL;
   }
   // Store source record
-  if (src_text) gt_string_set_nstring(src_text,line_start,buffered_input_file->cursor-line_start);
+  if (src_text) gt_string_set_nstring(src_text,line_start,buffered_map_input->cursor-line_start);
   // Next record
-  gt_input_map_parser_next_record(buffered_input_file);
+  gt_input_map_parser_next_record(buffered_map_input);
   return GT_IMP_OK;
 }
 GT_INLINE gt_status gt_input_map_parser_get_template(
-    gt_buffered_input_file* const buffered_input_file,gt_template* const template) {
-  GT_BUFFERED_INPUT_FILE_CHECK(buffered_input_file);
+    gt_buffered_input_file* const buffered_map_input,gt_template* const template) {
+  GT_BUFFERED_INPUT_FILE_CHECK(buffered_map_input);
   GT_TEMPLATE_CHECK(template);
-  return gt_imp_get_template(buffered_input_file,NULL,template,PARSE_ALL,NULL);
+  return gt_imp_get_template(buffered_map_input,NULL,template,PARSE_ALL);
 }
 GT_INLINE gt_status gt_input_map_parser_get_alignment(
-    gt_buffered_input_file* const buffered_input_file,gt_alignment* const alignment) {
-  GT_BUFFERED_INPUT_FILE_CHECK(buffered_input_file);
+    gt_buffered_input_file* const buffered_map_input,gt_alignment* const alignment) {
+  GT_BUFFERED_INPUT_FILE_CHECK(buffered_map_input);
   GT_ALIGNMENT_CHECK(alignment);
-  return gt_imp_get_alignment(buffered_input_file,NULL,alignment,PARSE_ALL,NULL);
+  return gt_imp_get_alignment(buffered_map_input,NULL,alignment,PARSE_ALL);
 }
 
 GT_INLINE gt_status gt_input_map_parser_get_template__src_text(
-    gt_buffered_input_file* const buffered_input_file,gt_template* const template,gt_string* const src_text) {
-  GT_BUFFERED_INPUT_FILE_CHECK(buffered_input_file);
+    gt_buffered_input_file* const buffered_map_input,gt_template* const template,gt_string* const src_text) {
+  GT_BUFFERED_INPUT_FILE_CHECK(buffered_map_input);
   GT_TEMPLATE_CHECK(template);
   GT_STRING_CHECK(src_text);
   gt_string_cast_static(src_text);
-  return gt_imp_get_template(buffered_input_file,src_text,template,PARSE_ALL,NULL);
+  return gt_imp_get_template(buffered_map_input,src_text,template,PARSE_ALL);
 }
 GT_INLINE gt_status gt_input_map_parser_get_alignment__src_text(
-    gt_buffered_input_file* const buffered_input_file,gt_alignment* const alignment,gt_string* const src_text) {
-  GT_BUFFERED_INPUT_FILE_CHECK(buffered_input_file);
+    gt_buffered_input_file* const buffered_map_input,gt_alignment* const alignment,gt_string* const src_text) {
+  GT_BUFFERED_INPUT_FILE_CHECK(buffered_map_input);
   GT_ALIGNMENT_CHECK(alignment);
   GT_STRING_CHECK(src_text);
   gt_string_cast_static(src_text);
-  return gt_imp_get_alignment(buffered_input_file,src_text,alignment,PARSE_ALL,NULL);
+  return gt_imp_get_alignment(buffered_map_input,src_text,alignment,PARSE_ALL);
 }
-
-
-// TODO
-//// Synchronized BMI. Attached to an output file & buffer,
-//// dumps the output whenever the end-of-input-block is reached
-//GT_INLINE gt_status gt_input_map_parser_get_template__sync_output(
-//    gt_buffered_input_file* const buffered_map_input,gt_template* template,gt_output_buffer** const output_buffer);
-//GT_INLINE gt_status gt_input_map_parser_get_alignment__sync_output(
-//    gt_buffered_input_file* const buffered_map_input,gt_alignment* alignment,gt_output_buffer** const output_buffer);
-//GT_INLINE gt_status gt_input_map_parser_get_template__sync_output(
-//    gt_buffered_input_file* const buffered_input_file,gt_template* template,gt_output_buffer** const output_buffer) {
-//  return gt_imp_get_template(buffered_input_file,template,PARSE_ALL,output_buffer);
-//}
-//GT_INLINE gt_status gt_input_map_parser_get_alignment__sync_output(
-//    gt_buffered_input_file* const buffered_input_file,gt_alignment* alignment,gt_output_buffer** const output_buffer) {
-//  return gt_imp_get_alignment(buffered_input_file,alignment,PARSE_ALL,output_buffer);
-//}
