@@ -9,6 +9,7 @@
 
 #include "gt_string.h"
 
+#define GT_STRING_STATIC 0
 #define GT_STRING_DEFAULT_BUFFER_SIZE 200
 
 /*
@@ -184,13 +185,19 @@ GT_INLINE bool gt_string_nequals(gt_string* const string_a,gt_string* const stri
 /*
  * Handlers
  */
-GT_INLINE gt_string* gt_string_copy(gt_string* const sequence) {
+GT_INLINE gt_string* gt_string_dup(gt_string* const sequence) {
   GT_STRING_CHECK(sequence);
   register gt_string* sequence_cpy = gt_string_new(sequence->length+1);
   gt_strncpy(sequence_cpy->buffer,sequence->buffer,sequence->length);
   sequence_cpy->length = sequence->length;
   return sequence_cpy;
 }
+GT_INLINE void gt_string_copy(gt_string* const sequence_dst,gt_string* const sequence_src) {
+  GT_STRING_CHECK_NO_STATIC(sequence_dst);
+  GT_STRING_CHECK(sequence_src);
+  gt_strncpy(sequence_dst->buffer,sequence_src->buffer,sequence_src->length);
+  sequence_dst->length = sequence_src->length;
+}
 GT_INLINE void gt_string_reverse_copy(gt_string* const sequence_dst,gt_string* const sequence_src) {
   GT_STRING_CHECK_NO_STATIC(sequence_dst);
   GT_STRING_CHECK(sequence_src);
@@ -217,3 +224,47 @@ GT_INLINE void gt_string_reverse(gt_string* const sequence) {
     buffer[string_length-i-1] = aux;
   }
 }
+
+/*
+ * String Printers
+ */
+GT_INLINE gt_status gt_vsprintf(gt_string* const sequence,const char *template,va_list v_args) {
+  GT_STRING_CHECK(sequence);
+  register gt_status chars_printed;
+  if (!gt_string_is_static(sequence)) { // Allocate memory
+    register const uint64_t mem_required = gt_calculate_memory_required_v(template,v_args);
+    gt_string_resize(sequence,mem_required+1);
+  }
+  chars_printed=vsprintf(gt_string_get_string(sequence),template,v_args);
+  gt_string_set_length(sequence,chars_printed);
+  gt_string_get_string(sequence)[chars_printed] = EOS;
+  return chars_printed;
+}
+GT_INLINE gt_status gt_sprintf(gt_string* const sequence,const char *template,...) {
+  GT_STRING_CHECK(sequence);
+  va_list v_args;
+  va_start(v_args,template);
+  register const gt_status chars_printed = gt_vsprintf(sequence,template,v_args);
+  va_end(v_args);
+  return chars_printed;
+}
+GT_INLINE gt_status gt_vsprintf_append(gt_string* const sequence,const char *template,va_list v_args) {
+  GT_STRING_CHECK(sequence);
+  register gt_status chars_printed = gt_string_get_length(sequence);
+  if (!gt_string_is_static(sequence)) { // Allocate memory
+    register const uint64_t mem_required = gt_calculate_memory_required_v(template,v_args);
+    gt_string_resize(sequence,mem_required+chars_printed+1);
+  }
+  chars_printed+=vsprintf(gt_string_get_string(sequence)+chars_printed,template,v_args);
+  gt_string_set_length(sequence,chars_printed);
+  gt_string_get_string(sequence)[chars_printed] = EOS;
+  return chars_printed;
+}
+GT_INLINE gt_status gt_sprintf_append(gt_string* const sequence,const char *template,...) {
+  GT_STRING_CHECK(sequence);
+  va_list v_args;
+  va_start(v_args,template);
+  register const gt_status chars_printed = gt_vsprintf_append(sequence,template,v_args);
+  va_end(v_args);
+  return chars_printed;
+}
