@@ -13,19 +13,6 @@
 #define GT_TEMPLATE_NUM_INITIAL_BLOCKS 2
 #define GT_TEMPLATE_NUM_INITIAL_MMAPS 20
 
-/*
- * Reduction to single alignment
- */
-#define GT_TEMPLATE_REDUCTION(template,reduced_alignment) \
-  register gt_alignment* const reduced_alignment = gt_template_get_block((template),0)
-#define GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template,reduced_alignment) \
-  if (gt_expect_false(gt_template_get_num_blocks((template))==1)) { \
-    GT_TEMPLATE_REDUCTION(template,reduced_alignment);
-#define GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT_(template) { \
-  if (gt_expect_false(gt_template_get_num_blocks((template))==1))
-#define GT_TEMPLATE_END_REDUCTION }
-#define GT_TEMPLATE_END_REDUCTION__RETURN return;}
-
 /*
  * Setup
  */
@@ -49,18 +36,26 @@ GT_INLINE void gt_template_clear_handler(gt_template* const template) {
   template->maps_txt = NULL;
   gt_shash_clean(template->attributes,true,true);
 }
-GT_INLINE void gt_template_clear(gt_template* const template,const bool clear_alignments) {
+GT_INLINE void gt_template_clear(gt_template* const template,const bool delete_alignments) {
   GT_TEMPLATE_CHECK(template);
-  if (clear_alignments) gt_template_delete_blocks(template);
+  if (delete_alignments) {
+    gt_template_delete_blocks(template);
+  }
   gt_vector_clean(template->counters);
   gt_vector_clean(template->mmaps);
   gt_vector_clean(template->mmaps_attributes);
   gt_template_clear_handler(template);
 }
-GT_INLINE void gt_template_delete(gt_template* const template,const bool delete_alignments) {
+GT_INLINE void gt_template_clear_alignments(gt_template* const template) {
+  GT_TEMPLATE_CHECK(template);
+  GT_VECTOR_ITERATE(template->blocks,alignment_it,alignment_pos,gt_alignment*) {
+    gt_alignment_clear(*alignment_it);
+  }
+}
+GT_INLINE void gt_template_delete(gt_template* const template) {
   GT_TEMPLATE_CHECK(template);
   gt_string_delete(template->tag);
-  if (delete_alignments) gt_template_delete_blocks(template);
+  gt_template_delete_blocks(template);
   gt_vector_delete(template->blocks);
   gt_vector_delete(template->counters);
   gt_vector_delete(template->mmaps);
@@ -81,6 +76,17 @@ GT_INLINE void gt_template_set_tag(gt_template* const template,char* const tag,c
   GT_NULL_CHECK(tag);
   gt_string_set_nstring(template->tag,tag,length);
 }
+GT_INLINE uint64_t gt_template_get_total_length(gt_template* const template) {
+  GT_TEMPLATE_CHECK(template);
+  GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template,alignment) {
+    return gt_alignment_get_read_length(alignment);
+  } GT_TEMPLATE_END_REDUCTION;
+  register uint64_t total_length = 0;
+  GT_TEMPLATE_ALIGNMENT_ITERATE(template,alignment) {
+    total_length += gt_alignment_get_read_length(alignment);
+  }
+  return total_length;
+}
 /* Blocks (single alignments) */
 GT_INLINE uint64_t gt_template_get_num_blocks(gt_template* const template) {
   GT_TEMPLATE_CHECK(template);
@@ -149,13 +155,21 @@ GT_INLINE uint64_t gt_template_get_counter(gt_template* const template,const uin
   } GT_TEMPLATE_END_REDUCTION;
   return *gt_vector_get_elm(template->counters,stratum-1,uint64_t);
 }
+GT_INLINE void gt_template_dynamically_allocate_counter(gt_template* const template,const uint64_t stratum) {
+  GT_TEMPLATE_CHECK(template);
+  gt_fatal_check(stratum==0,COUNTERS_POS_STRATUM);
+  gt_vector_reserve(template->counters,stratum,true);
+  if (gt_vector_get_used(template->counters)<stratum) {
+    gt_vector_set_used(template->counters,stratum);
+  }
+}
 GT_INLINE void gt_template_inc_counter(gt_template* const template,const uint64_t stratum) {
   GT_TEMPLATE_CHECK(template);
   gt_fatal_check(stratum==0,COUNTERS_POS_STRATUM);
   GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template,alignment) {
     gt_alignment_inc_counter(alignment,stratum);
   } GT_TEMPLATE_END_REDUCTION__RETURN;
-  gt_vector_reserve(template->counters,stratum,true);
+  gt_template_dynamically_allocate_counter(template,stratum);
   ++(*gt_vector_get_elm(template->counters,stratum-1,uint64_t));
 }
 GT_INLINE void gt_template_dec_counter(gt_template* const template,const uint64_t stratum) {
@@ -164,7 +178,7 @@ GT_INLINE void gt_template_dec_counter(gt_template* const template,const uint64_
   GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template,alignment) {
     gt_alignment_dec_counter(alignment,stratum);
   } GT_TEMPLATE_END_REDUCTION__RETURN;
-  gt_vector_reserve(template->counters,stratum,true);
+  gt_template_dynamically_allocate_counter(template,stratum);
   --(*gt_vector_get_elm(template->counters,stratum-1,uint64_t));
 }
 GT_INLINE void gt_template_set_counter(gt_template* const template,const uint64_t stratum,const uint64_t value) {
@@ -173,7 +187,7 @@ GT_INLINE void gt_template_set_counter(gt_template* const template,const uint64_
   GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template,alignment) {
     gt_alignment_set_counter(alignment,stratum,value);
   } GT_TEMPLATE_END_REDUCTION__RETURN;
-  gt_vector_reserve(template->counters,stratum,true);
+  gt_template_dynamically_allocate_counter(template,stratum);
   *gt_vector_get_elm(template->counters,stratum-1,uint64_t) = value;
 }
 
@@ -205,7 +219,7 @@ GT_INLINE void gt_template_set_attr(
 /*
  * Predefined attributes
  */
-GT_INLINE uint64_t gt_template_get_mcs(gt_template* const template) {
+GT_INLINE uint64_t gt_template_get_mcs(gt_template* const template) { // FIXME: reduce
   GT_TEMPLATE_CHECK(template);
   register uint64_t* mcs = gt_template_get_attribute(template,GT_ATTR_MAX_COMPLETE_STRATA,uint64_t);
   if (mcs == NULL) return UINT64_MAX;
@@ -394,58 +408,46 @@ GT_INLINE void gt_template_copy_handler(gt_template* template_dst,gt_template* c
   GT_TEMPLATE_CONSISTENCY_CHECK(template_src);
   template_dst->template_id = template_src->template_id;
   template_dst->in_block_id = template_src->in_block_id;
-  template_dst->tag = template_src->tag;
-  gt_vector_copy(template_dst->blocks,template_src->blocks);
+  gt_string_copy(template_dst->tag,template_src->tag);
   template_dst->maps_txt = template_src->maps_txt;
+  // Copy templates' attributes
+  template_dst->attributes = gt_shash_deep_copy(template_src->attributes);
 }
-GT_INLINE void gt_template_copy_blocks(gt_template* template_dst,gt_template* const template_src,const bool deep_copy) {
+GT_INLINE void gt_template_copy_blocks(gt_template* template_dst,gt_template* const template_src,const bool copy_maps) {
   GT_TEMPLATE_CONSISTENCY_CHECK(template_src);
   gt_vector_clean(template_dst->blocks);
   register const uint64_t num_blocks = gt_template_get_num_blocks(template_src);
   register uint64_t i;
   for (i=0;i<num_blocks;++i) {
-    register gt_alignment* alg_copy = gt_expect_false(deep_copy) ?
-        gt_alignment_deep_copy(gt_template_get_block(template_src,i)) :
-        gt_alignment_copy(gt_template_get_block(template_src,i)) ;
-    gt_template_add_block(template_dst,alg_copy);
+    register gt_alignment* alignment_copy =
+        gt_alignment_copy(gt_template_get_block(template_src,i),copy_maps);
+    gt_template_add_block(template_dst,alignment_copy);
   }
 }
-GT_INLINE gt_template* gt_template_copy(gt_template* const template) {
-  GT_TEMPLATE_CONSISTENCY_CHECK(template);
-  gt_template* template_cpy = gt_template_new();
-  gt_cond_fatal_error(!template_cpy,MEM_HANDLER);
-  // Copy handler
-  gt_template_copy_handler(template_cpy,template);
-  return template_cpy;
-}
-GT_INLINE gt_template* gt_template_deep_copy(gt_template* const template) {
+GT_INLINE gt_template* gt_template_copy(gt_template* const template,const bool copy_maps,const bool copy_mmaps) {
   GT_TEMPLATE_CONSISTENCY_CHECK(template);
   gt_template* template_cpy = gt_template_new();
   gt_cond_fatal_error(!template_cpy,MEM_HANDLER);
   // Copy handler
   gt_template_copy_handler(template_cpy,template);
   // Copy blocks
-  gt_template_copy_blocks(template_cpy,template,true);
-  // Copy counters
-  gt_vector_copy(template_cpy->counters,template->counters);
-  // Copy mmaps & mmaps_attributes
-  gt_map** mmap;
-  gt_mmap_attributes* mmap_attr;
-  gt_template_maps_iterator template_maps_iterator;
-  gt_template_new_mmap_iterator(template,&template_maps_iterator);
-  register const uint64_t num_blocks = gt_template_get_num_blocks(template);
-  while (gt_expect_true(gt_template_next_mmap(&template_maps_iterator,&mmap,&mmap_attr))) {
-    uint64_t i, map_pos;
-    for (i=0;i<num_blocks;++i) {
-      gt_cond_fatal_error(!gt_alignment_locate_map_reference(
-          gt_template_get_block(template,i),mmap[i],&map_pos),TEMPLATE_INCONSISTENT_MMAPS_ALIGNMENT);
-      register gt_map* homologe_map = gt_alignment_get_map(gt_template_get_block(template_cpy,i),map_pos);
-      gt_vector_insert(template->mmaps,homologe_map,gt_map*);
+  gt_template_copy_blocks(template_cpy,template,copy_maps);
+  // Copy mmaps
+  if (copy_maps && copy_mmaps) {
+    // Copy counters
+    gt_vector_copy(template_cpy->counters,template->counters);
+    // Copy mmaps & mmaps_attributes
+    GT_TEMPLATE__ATTR_ITERATE(template,mmap,mmap_attr) {
+      uint64_t map_pos;
+      GT_MULTIMAP_ITERATE(mmap,map,end_pos) {
+        gt_cond_fatal_error(!gt_alignment_locate_map_reference(
+            gt_template_get_block(template,end_pos),map,&map_pos),TEMPLATE_INCONSISTENT_MMAPS_ALIGNMENT);
+        register gt_map* homologe_map = gt_alignment_get_map(gt_template_get_block(template_cpy,end_pos),map_pos);
+        gt_vector_insert(template_cpy->mmaps,homologe_map,gt_map*);
+      }
+      gt_vector_insert(template_cpy->mmaps_attributes,*mmap_attr,gt_mmap_attributes);
     }
-    gt_vector_insert(template->mmaps_attributes,*mmap_attr,gt_mmap_attributes);
   }
-  // Copy templates' attributes
-  template_cpy->attributes = gt_shash_deep_copy(template->attributes);
   return template_cpy;
 }
 
