@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 """Python wrapper around the GEM2 mapper that provides
-ability to feed data into GEM and retreive the mappings"""
+ability to feed data into GEM and retrieve the mappings"""
 import os
 import shutil
 import sys
@@ -16,6 +16,7 @@ from . import utils
 import pkg_resources
 import splits
 import filter as gemfilter
+import gem.gemtools as gt
 
 LOG_NOTHING = 1
 LOG_STDERR = 2
@@ -35,6 +36,10 @@ default_filter = "same-chromosome,same-strand"
 
 ## use the bundled executables
 use_bundled_executables = True
+## max mappings to replace mapping counts for + and ! summaries
+__max_mappings = 999999999
+## filter to work around GT-32 and #006 in gem-map-2-map 
+__awk_filter = ["awk", "-F", "\t", '{if($4 == "*" || $4 == "-"){print $1"\t"$2"\t"$3"\t0\t"$5}else{if($4 == "!" || $4 == "+"){print $1"\t"$2"\t"$3"\t'+str(__max_mappings)+'\t"$5}else{print}}}']
 
 class execs_dict(dict):
     """Helper dict that resolves bundled binaries"""
@@ -95,6 +100,8 @@ class Read(object):
         self.mappings = None
         self.line = None
         self.type = None
+        self.template = None
+        self.__template_initialized = False
 
     def fill(self, other):
         """Fill this read with the content of another read"""
@@ -105,6 +112,8 @@ class Read(object):
         self.mappings = other.mappings
         self.line = other.line
         self.type = other.type
+        self.template = None
+        self.__template_initialized = False
 
     def min_mismatches(self):
         """Parse the mismatch string and return the minimum number
@@ -126,15 +135,26 @@ class Read(object):
 
 
     def get_maps(self):
-        if self.summary == None or self.summary in ['-', '+', '*']:
+        if self.summary == None or self.summary in ['-', '*']:
             return ([0], ["-"])
+        elif self.summary in ['+', '!']:
+            return ([__max_mappings], ["-"])
+
         sums = [int(x) for x in utils.multisplit(self.summary, [':', '+'])]
         maps = self.mappings.split(',')
         return (sums, maps)
 
 
+    def _get_template(self):
+        if self.template is None:
+            self.template = gt.Template()
+        if not self.__template_initialized:
+            self.template.fill(self.line)
+        return self.template
+
+
     def __str__(self):
-        """Returns GEM string representaton of the reads"""
+        """Returns GEM string representation of the reads"""
         if self.qualities is None:
             return "%s\t%s\t%s\t%s" % (self.id,
                                        self.sequence,
@@ -339,6 +359,7 @@ def mapper(input, index, output=None,
            min_matched_bases=0.80,
            max_big_indel_length=15,
            max_edit_distance=0.20,
+           mismatch_alphabet="ACGT",
            trim=None,
            threads=1):
     """Start the GEM mapper on the given input.
@@ -383,6 +404,7 @@ def mapper(input, index, output=None,
           '--min-matched-bases', str(min_matched_bases),
           '--gem-quality-threshold', str(quality_threshold),
           '--max-big-indel-length', str(max_big_indel_length),
+          '--mismatch-alphabet', mismatch_alphabet,
           '-T', str(threads)
     ]
 
@@ -397,12 +419,18 @@ def mapper(input, index, output=None,
             raise ValueError("Trim parameter has to be a list or a tuple of size 2")
         input = gemfilter.trim(input, trim[0], trim[1], append_label=True)
 
+    # workaround for GT-32 - filter away the !
+    # build list of tools
+    tools = [pa, __awk_filter]
+    if trim is not None:
+        tools.append(trim_c)
+
     ## run the mapper
     process = None
-    if trim is None:
-        process = utils.run_tool(pa, input, output, "GEM-Mapper", utils.read_to_sequence)
+    if len(tools) == 1:
+        process = utils.run_tool(tools[0], input, output, "GEM-Mapper", utils.read_to_sequence)
     else:
-        process = utils.run_tools([pa, trim_c], input, output, "GEM-Mapper", utils.read_to_sequence)
+        process = utils.run_tools(tools, input, output, "GEM-Mapper", utils.read_to_sequence)
 
     return _prepare_output(process, output, type="map", name="GEM-Mapper", quality=quality)
 
@@ -421,6 +449,8 @@ def splitmapper(input,
                 mismatch_strata_delta=1,
                 quality=33,
                 trim=None,
+                post_validate=True,
+                mismatch_alphabet="ACGT",
                 threads=1):
     """Start the GEM split mapper on the given input.
     If input is a file handle, it is assumed to
@@ -453,6 +483,7 @@ def splitmapper(input,
           '--refinement-step-size', str(refinement_step_size),
           '--matches-threshold', str(matches_threshold),
           '--strata-after-first', str(mismatch_strata_delta),
+          '--mismatch-alphabet', mismatch_alphabet,
           '-T', str(threads)
     ]
 
@@ -475,13 +506,26 @@ def splitmapper(input,
             raise ValueError("Trim parameter has to be a list or a tuple of size 2")
         input = gemfilter.trim(input, trim[0], trim[1], append_label=True)
 
+    tools = [pa, __awk_filter]
+    if trim is not None:
+        tools.append(trim_c)
+
     ## run the mapper
     process = None
-    if trim is None:
-        process = utils.run_tool(pa, input, output, "GEM-Split-Mapper", utils.read_to_sequence)
+    original_output = output
+    if post_validate:
+        output = None
+    if len(tools) == 1:
+        process = utils.run_tool(tools[0], input, output, "GEM-Split-Mapper", utils.read_to_sequence)
     else:
-        process = utils.run_tools([pa, trim_c], input, output, "GEM-Split-Mapper", utils.read_to_sequence)
-    return _prepare_output(process, output, type="map", name="GEM-Split-Mapper", quality=quality)
+        process = utils.run_tools(tools, input, output, "GEM-Split-Mapper", utils.read_to_sequence)
+
+    splitmap_out = _prepare_output(process, output, type="map", name="GEM-Split-Mapper", quality=quality)
+
+    if post_validate:
+        return validate(splitmap_out, index, original_output, threads=threads)
+
+    return splitmap_out
 
 
 def extract_junctions(input,
@@ -513,6 +557,7 @@ def extract_junctions(input,
         matches_threshold=matches_threshold,
         splice_consensus=splice_consensus,
         quality=quality,
+        post_validate=False,
         threads=threads)
     ## make sure we have an output file
     ## for the splitmap results
@@ -621,18 +666,22 @@ def realign(input,
 def validate(input,
              index,
              output=None,
-             validate_score="-s,-b,-i",
-             validate_filter="2,25",
+             validate_score=None, # "-s,-b,-i"
+             validate_filter=None, # "2,25"
              threads=1, ):
     index = _prepare_index_parameter(index, gem_suffix=False)
     validate_p = [executables['gem-map-2-map'],
                   '-I', index,
                   '-v', '-r',
-                  '-s', validate_score,
-                  '-f', validate_filter,
                   '-T', str(max(threads, 1))
     ]
 
+    if validate_score is not None:
+        validate_p.extend(["-s", validate_score])
+
+    if validate_filter is not None:
+        validate_p.extend(['-f', validate_filter])
+
     quality = None
     if isinstance(input, files.ReadIterator):
         quality = input.quality
@@ -672,7 +721,7 @@ def validate_and_score(input,
 
 def gem2sam(input, index=None, output=None, single_end=False, compact=False, threads=1, quality=None, check_ids=True):
     if index is not None:
-        index = _prepare_index_parameter(index, False)
+        index = _prepare_index_parameter(index, True)
     gem_2_sam_p = [executables['gem-2-sam'],
                    '-T', str(threads)
     ]
